package com.lilosoft.api.service.complexWindow;

import cn.hutool.core.io.resource.FileObjectResource;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.lilosoft.api.bean.receive.FileDataItem;
import com.lilosoft.complex.Rest;
import com.lilosoft.complex.matter.model.req.*;
import com.lilosoft.complex.matter.model.rsp.AcceptRspInfo;
import com.lilosoft.complex.matter.model.rsp.LoginRspInfo;
import com.lilosoft.complex.matter.model.rsp.MatterListInfo;
import com.lilosoft.complex.matter.model.rsp.MatterRespInfo;
import com.lilosoft.complex.matter.model.rsp.matter.ChildrenItem;
import com.lilosoft.complex.matter.model.rsp.matter.MatterWorkInfo;
import com.lilosoft.complex.matter.service.IComplexMatterService;
import com.lilosoft.core.enums.TypeEnum;
import com.lilosoft.core.enums.YesNoEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 危险化学品经营许可法人变更
 *
 * @author:
 * @date: 2023/7/31 15:15
 */
@Slf4j
public class YLJGZYBGFRBG extends ComplexWindowAbstract {

    private IComplexMatterService complexMatterService;

    public static String matterName = "医疗机构执业变更";


    public YLJGZYBGFRBG(String type) {
        super(type);
        this.complexMatterService = SpringUtil.getBean(IComplexMatterService.class);

    }

    @Override
    String getType() {
        return TypeEnum.医疗机构执业变更法人变更.getValue();
    }

    @Override
    public void doHandleWork(CommonData commonData) {
        log.info("commonData = " + JSON.toJSONString(commonData));
        //下载附件
        List<FileDataItem> fileCollect = commonData.getFileData().stream().map(item -> {
            FileDataItem fileDataItem = new FileDataItem();
            BeanUtils.copyProperties(item, fileDataItem);
            String downloadPath = commonData.getRobertServerUrl()+"aiCase/download?fileUrl=" + item.getFileUrl();
            byte[] bytes = HttpUtil.downloadBytes(downloadPath);
            fileDataItem.setBytes(bytes);
            return fileDataItem;
        }).collect(Collectors.toList());

        //通过名称查询
        DeviceLoginReq deviceLoginReq = new DeviceLoginReq();
        deviceLoginReq.setDeviceLogo(commonData.getDeviceLogo());
        Rest<LoginRspInfo> loginRspInfoRest = complexMatterService.deviceLogin(deviceLoginReq);

        if (loginRspInfoRest.getCode() == YesNoEnum.YES.getValue()) {
            //设备登录成功 上传附件
            fileCollect.stream().forEach(item -> {
                Rest<String> uploadFileRest = complexMatterService.uploadFile(commonData.getDeviceLogo(), item.getBytes(), item.getFileName());
                if (uploadFileRest.getCode() == YesNoEnum.YES.getValue()) {
                    item.setDownloadUrl(uploadFileRest.getData());
                }
            });

            //通过事项查询eventId
            ImplementationReq implementationReq = new ImplementationReq();
            implementationReq.setImplementName(matterName);
            Rest<MatterListInfo> matterRest = complexMatterService.findMatters(commonData.getDeviceLogo(), implementationReq);
            log.info("matterRest = " + JSON.toJSONString(matterRest));
            if (matterRest.getCode() == YesNoEnum.YES.getValue()) {
                List<MatterRespInfo> matterListInfos = matterRest.getData().getRecords();
                MatterRespInfo matterRespInfo = null;
                if (!ObjectUtils.isEmpty(matterListInfos)) {
                    matterRespInfo =  matterListInfos.stream().filter(f -> "11511200MB1887563L300012300400009".equals(f.getHandlingItemCode())).findFirst().orElseGet(() -> null);

                    if(matterRespInfo != null){
                        String eventId = matterRespInfo.getEventId();
                        String eventObjectType = matterRespInfo.getEventObjectType();

                        AcceptReq acceptReq = new AcceptReq();

                        //查询办理事项情形
                        Rest<MatterWorkInfo> matterWorkInfoRest = complexMatterService.doWorkMatterSearch(commonData.getDeviceLogo(), eventId);
                        log.info("matterWorkInfoRest = " + JSON.toJSONString(matterWorkInfoRest));
                        if (matterWorkInfoRest.getCode() == YesNoEnum.YES.getValue()) {
                            MatterWorkInfo matterWorkInfo = matterWorkInfoRest.getData();
                            ChildrenItem childrenItem = matterWorkInfo.getChildren().stream().filter(f -> "医疗机构执业许可证变更法定代表人及负责人".equals(f.getLabel())).findFirst().orElseGet(() -> null);

                            if (!ObjectUtils.isEmpty(childrenItem)) {
                                String situationItemId = childrenItem.getSituationItemId();
                                acceptReq.getItemsIds().add(situationItemId);

                                //构建受理材料附件
                                List<MaterialsItem> materialsItemList = childrenItem.getMaterials().stream().map(item -> {
                                    MaterialsItem materialsItem = new MaterialsItem();
                                    String name = item.getName();
                                    if (StrUtil.contains(name, "负责人医师资格证")) {
                                        matchUploadFile(fileCollect, materialsItem, "申请变更登记情况页");
                                    } else if (StrUtil.contains(name, "履历表")) {
                                        matchUploadFile(fileCollect, materialsItem, "上级主管部门意见页");
                                    } else if (StrUtil.contains(name, "变更登记注册书")) {
                                        matchUploadFile(fileCollect, materialsItem, "医疗机构变更登记页");
                                    } else if (StrUtil.contains(name, "任免职证明文件")) {
                                        matchUploadFile(fileCollect, materialsItem, "核准变更登记事项页");
                                    } else if (StrUtil.contains(name, "法定代表人签字表")) {
                                        matchUploadFile(fileCollect, materialsItem, "核发医疗机构执业许可证");
                                    } else if (StrUtil.contains(name, "医疗机构执业许可证")) {
                                        matchUploadFile(fileCollect, materialsItem, "许可证副本");
                                    }


                                    materialsItem.setMaterialForm(item.getFromId());
                                    materialsItem.setMaterialId(item.getId());
                                    return materialsItem;

                                }).collect(Collectors.toList());
                                acceptReq.setMaterials(materialsItemList);
                            }
                        }
                        //提交响应的受理
                        List<HandlingsItem> handlingsItemList = new ArrayList<>();
                        HandlingsItem handlingsItem = new HandlingsItem();
                        handlingsItem.setEventId(eventId);
                        handlingsItem.setObjectType(eventObjectType);
                        handlingsItemList.add(handlingsItem);
                        acceptReq.setHandlings(handlingsItemList);
                        //申请人信息
                        ApplicantLeagal applicantLeagal = new ApplicantLeagal();
                        applicantLeagal.setPhone(commonData.getApplicantPhone());
                        applicantLeagal.setApplicantSex(commonData.getApplicantSex());
                        applicantLeagal.setDocumentId(commonData.getApplicantDocumentId());
                        applicantLeagal.setApplicantName(commonData.getApplicantName());
                        applicantLeagal.setApplicantIdNumber(commonData.getApplicantIdNumber());
                        acceptReq.setApplicantLegal(applicantLeagal);
                        //法人信息
                        LegalProxy legalProxy = new LegalProxy();
                        legalProxy.setLegalName(commonData.getLegalName());
                        legalProxy.setLegalIdNumber(commonData.getLegalIdNumber());
                        legalProxy.setDocumentId(commonData.getLegalDocumentId());
                        acceptReq.setLegalProxy(legalProxy);
                        Rest<List<AcceptRspInfo>> acceptRest = complexMatterService.accept(commonData.getDeviceLogo(), acceptReq);

                        log.info("受理返回==>{}", JSON.toJSONString(acceptRest));
                        if (acceptRest.getCode() == YesNoEnum.YES.getValue()) {
                            log.info("操作成功==>{}", JSON.toJSONString(acceptRest));
                        }
                    }else {
                        log.error("未查询到匹配事项 ==>{}",JSON.toJSONString(matterRest));
                    }

                }else {
                    log.error("返回事项列表为空 ==>{}",JSON.toJSONString(matterRest));
                }


            }

        }

    }

    private static void matchUploadFile(List<FileDataItem> fileCollect, MaterialsItem materialsItem, String fileName) {
        FileDataItem fileDataItem = fileCollect.stream().filter(f -> StrUtil.contains(f.getName(), fileName)).findFirst().orElseGet(() -> null);
        if (!ObjectUtils.isEmpty(fileDataItem)) {
            materialsItem.setUploadFileName(fileDataItem.getFileName());
            materialsItem.setMaterialPath(fileDataItem.getDownloadUrl());
        }
    }
}
