package com.lilosoft.core.executor.structs;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 消息队列
 */
public class MessageQueue<V> {
	
//	private static int MAX_TASK = 100;

	private Lock lock = new ReentrantLock();
//	private Condition notFull = lock.newCondition();  
	private Condition notEmpty = lock.newCondition();   
	
	private final LinkedBlockingQueue<V> messageQueue = new LinkedBlockingQueue<V>();

	private boolean processingCompleted = true;

	/**
	 * 下一执行命令
	 * 
	 * @return
	 */
	public V poll() throws InterruptedException {  
		try {
			lock.lock();
			while(0 == messageQueue.size()) {  
                notEmpty.await();  
            }  
			V obj = messageQueue.poll();
//			notFull.signalAll();  
			return obj;
		} finally {
			lock.unlock();
		}
	}

	/**
	 * 增加执行指令
	 * @return
	 */
	public boolean add(V value) throws InterruptedException {  
		try {
			lock.lock();
//			while (MAX_TASK == messageQueue.size()) {  
//                notFull.await();  
//            } 
			boolean ret = messageQueue.add(value);
			notEmpty.signalAll();  
			return ret;
		} finally {
			lock.unlock();
		}
	}

	/**
	 * 清理
	 */
	public void clear() {
		try {
			lock.lock();
			messageQueue.clear();
		} finally {
			lock.unlock();
		}
	}

	/**
	 * 获取指令数量
	 * 
	 * @return
	 */
	public int size() {
		return messageQueue.size();
	}

	public boolean isProcessingCompleted() {
		return processingCompleted;
	}

	public void setProcessingCompleted(boolean processingCompleted) {
		this.processingCompleted = processingCompleted;
	}

}
