package com.lilosoft.core.queue.service;

import com.lilosoft.core.queue.command.Command;

import java.util.concurrent.LinkedBlockingQueue;

/** 
 * @ClassName: SystemQueueService 
 * @Description: 系统队列服务
 * @author: YanJie
 */
public class SystemQueueService extends Thread {
	private boolean status = true;
	protected static SystemQueueService instance = new SystemQueueService();

	private final LinkedBlockingQueue<Command> queue = new LinkedBlockingQueue<Command>(100);

	public synchronized static SystemQueueService getInstance() {
		if (instance == null) {
			instance = new SystemQueueService();
		}
		return instance;
	}
	
	@Override
	public void run() {
		while(status){
			execute();
		}
	}
	
	/**
	 * 向短信队列推送待发送短信消息体
	 */
	public void putCommand(Command command) {
		try{
			/**
			 * LinkedBlockingQueue.put()：首选。队满是阻塞
			 * LinkedBlockingQueue.offer()：队满时返回false
			 */
			queue.put(command);
//			if (queue.isProcessingCompleted()) {
//				queue.setProcessingCompleted(false);
//			}
			System.out.println(queue.size());
		} catch (Exception e) {	
			e.printStackTrace();
		}
	}

	/**
	 * 获取短信队列未发送短信数量
	 * @return
	 */
	public int getQueueSize(){
		return queue.size();
	}
	
	private void execute(){
		try {
			/**
			 * LinkedBlockingQueue.take()：首选。当队列为空时阻塞
			 * LinkedBlockingQueue.poll()：弹出队顶元素，队列为空时，返回空
			 */
			Command command = queue.take();
			if (command != null) {
				command.execute();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void close(){
		status = false;
	}
}
